@@ -1,7 +1,7 @@
 !
 !  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 !  SLEPc - Scalable Library for Eigenvalue Problem Computations
-!  Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+!  Copyright (c) 2002-2019, Universitat Politecnica de Valencia, Spain
 !
 !  This file is part of SLEPc.
 !  SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -9,8 +9,8 @@
 !
 !  Include file for Fortran use of the BV object in SLEPc
 !
-#if !defined(__SLEPCBVDEF_H)
-#define __SLEPCBVDEF_H
+#if !defined(SLEPCBVDEF_H)
+#define SLEPCBVDEF_H
 
 #include "petsc/finclude/petscmat.h"
 
@@ -26,5 +26,6 @@
 #define BVSVEC       'svec'
 #define BVVECS       'vecs'
 #define BVCONTIGUOUS 'contiguous'
+#define BVTENSOR     'tensor'
 
 #endif
