@@ -1,7 +1,7 @@
 !
 !  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 !  SLEPc - Scalable Library for Eigenvalue Problem Computations
-!  Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+!  Copyright (c) 2002-2019, Universitat Politecnica de Valencia, Spain
 !
 !  This file is part of SLEPc.
 !  SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -9,8 +9,9 @@
 !
 !  Basic include file for Fortran use of the SLEPc package
 !
-#if !defined(__SLEPCSYSDEF_H)
-#define __SLEPCSYSDEF_H
+#if !defined(SLEPCSYSDEF_H)
+#define SLEPCSYSDEF_H
+
 #include "petscconf.h"
 #include "petsc/finclude/petsc.h"
 #include "slepcversion.h"
