@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2019, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -115,3 +115,21 @@ int main(int argc,char **argv)
   return ierr;
 }
 
+/*TEST
+
+   testset:
+      args: -mfn_type {{krylov expokit}}
+      output_file: output/test2_1.out
+      test:
+         suffix: 1
+      test:
+         suffix: 1_cuda
+         args: -mat_type aijcusparse
+         requires: cuda
+
+   test:
+      suffix: 3
+      args: -mfn_type expokit -t 0.6
+      requires: !__float128
+
+TEST*/
